#ifndef SLAM_UTIL_TOOL_H
#define SLAM_UTIL_TOOL_H

#include <iostream>
#include <cmath>
#include <vector>
#include "interface/common_type.h"
#include <limits>
#include <util/nanoflann.h>
#include <cstdlib>

using namespace nanoflann;

namespace slam
{
    class SlamUtil
    {

    public:
        template <typename T>
        struct PointCloud
        {
            const std::vector<slam::Vector2d>& pts;

            PointCloud(const std::vector<slam::Vector2d>& pts) : pts(pts) {}

            inline size_t kdtree_get_point_count() const { return pts.size(); }

            inline T kdtree_get_pt(const size_t idx, int dim) const
            {
                if (dim == 0) return pts[idx].x;
                else return pts[idx].y;
            }

            template <class BBOX>
            bool kdtree_get_bbox(BBOX& /* bb */) const { return false; }
        };

        typedef KDTreeSingleIndexAdaptor<
            L2_Simple_Adaptor<double, PointCloud<double> >,
            PointCloud<double>,
            2 /* dim */
        > slam_kd_tree_t;

        static Quaterniond EulerToQuaternion(double roll, double pitch, double yaw);

        static int findNearestPointIndex(const slam::Vector2d& cur_host_pos,
                                     const std::vector<slam::Vector2d>& ref_line,
                                     double max_distance, double min_distance);

        static int findNearestPointIndex(const slam::Vector2d& cur_host_pos,
                                         const std::vector<slam::Vector2d>& ref_line);

        static slam::Vector2d globalToLocal(slam::Vector2d glb_p,slam::Vector2d center_p, double heading);
        static slam::Vector2d gcs2vcs(slam::Vector2d glb_p,slam::Vector2d center_p, double heading);
        static slam_kd_tree_t* buildKDTree(const std::vector<slam::Vector2d>& ref_line);

        static int findNearest(slam_kd_tree_t* index, const slam::Vector2d& query_point, double& out_min_dist);

    private:

    };

} // namespace slam

#endif // SLAM_UTIL_TOOL_H
